/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.api;

import java.io.IOException;
import java.io.Serializable;

import com.hazelcast.nio.serialization.Portable;
import com.hazelcast.nio.serialization.PortableReader;
import com.hazelcast.nio.serialization.PortableWriter;

/**
 * This classes is used to store data into the registry without depending on too much petals specific classes.
 * 
 * It also enables us to take advantage of {@link Portable} implementation which improves performance of
 * serialization/deserializaton as well as indexing and querying compared to basic Java {@link Serializable}.
 * 
 * @author vnoel
 *
 */
public class StoredRegistryEndpoint implements Portable {

    public enum Type {
        INTERNAL, EXTERNAL;
    }

    private Type type;

    private String endpointName;

    private String serviceName;

    private String[] interfaces;

    private String container;

    private String component;

    private boolean inRestoration;

    StoredRegistryEndpoint() {
        // used by RegistryEndpointFactory
    }

    public StoredRegistryEndpoint(final Type type, final String endpointName, final String serviceName,
            final String[] interfaces, final String container, final String component) {
        this.type = type;
        this.endpointName = endpointName;
        this.serviceName = serviceName;
        this.interfaces = interfaces;
        this.container = container;
        this.component = component;
        this.inRestoration = false;
    }

    public StoredRegistryEndpoint(final StoredRegistryEndpoint endpoint, final boolean inRestoration) {
        this(endpoint.getType(), endpoint.getEndpointName(), endpoint.getServiceName(), endpoint.getInterfaces(),
                endpoint.getContainer(), endpoint.getComponent());
        this.inRestoration = inRestoration;
    }

    @Override
    public int getFactoryId() {
        return PetalsRegistryPortableFactory.FACTORY_ID;
    }

    @Override
    public int getClassId() {
        return PetalsRegistryPortableFactory.STORED_REGISTRY_ENDPOINT_CLASS_ID;
    }

    @Override
    public void writePortable(final PortableWriter writer) throws IOException {
        writer.writeUTF(DataModelApi.ENDPOINT_TYPE_FIELD, type.name());
        writer.writeUTF(DataModelApi.ENDPOINT_NAME_FIELD, endpointName);
        writer.writeUTF(DataModelApi.ENDPOINT_SERVICE_FIELD, serviceName);
        writer.writeUTFArray(DataModelApi.ENDPOINT_INTERFACES_FIELD, interfaces);
        writer.writeUTF(DataModelApi.ENDPOINT_CONTAINER_FIELD, container);
        writer.writeUTF(DataModelApi.ENDPOINT_COMPONENT_FIELD, component);
        writer.writeBoolean(DataModelApi.ENDPOINT_RESTORATION_FIELD, inRestoration);
    }

    @Override
    public void readPortable(final PortableReader reader) throws IOException {
        type = Type.valueOf(reader.readUTF(DataModelApi.ENDPOINT_TYPE_FIELD));
        endpointName = reader.readUTF(DataModelApi.ENDPOINT_NAME_FIELD);
        serviceName = reader.readUTF(DataModelApi.ENDPOINT_SERVICE_FIELD);
        interfaces = reader.readUTFArray(DataModelApi.ENDPOINT_INTERFACES_FIELD);
        container = reader.readUTF(DataModelApi.ENDPOINT_CONTAINER_FIELD);
        component = reader.readUTF(DataModelApi.ENDPOINT_COMPONENT_FIELD);
        inRestoration = reader.readBoolean(DataModelApi.ENDPOINT_RESTORATION_FIELD);
    }

    public Type getType() {
        return type;
    }

    public String getEndpointName() {
        return endpointName;
    }

    public String getServiceName() {
        return serviceName;
    }

    public String[] getInterfaces() {
        return interfaces;
    }

    public String getContainer() {
        return container;
    }

    public String getComponent() {
        return component;
    }

    public boolean isInRestoration() {
        return inRestoration;
    }
}
